
# c2py.py

from cmm_parser import *
from output import Output

# --------------------------------------------------------------------------

class C2Py (Output) :

   def send_base_name (self, param) :
       if param.spec_func != None :
          self.send_special_function (param.spec_func)
       elif param.id != "" :
          self.send (param.id)

   def send_cont_item (self, param) :
       self.style_no_space ()
       self.send (".")
       self.style_no_space ()
       self.send_cont_name (param)

   def send_cont_name (self, param) :
       if param.spec_func != None :
          self.send_special_function (param.spec_func)
       elif param.id != "" :
          self.send (param.id)

   def send_simple_name (self, param) :
       self.send (param.id)

   def send_qualified_name (self, param) :
       self.send_base_name (param)
       for item in param.items :
          self.send_cont_item (item)

# --------------------------------------------------------------------------

   def send_string_literal_cont (self, param) :
       self.send ('"' + param.value + '"')

   def send_expr_list (self, param) :
       first = True
       for item in param.items :
           if not first :
              self.send (",")
           first = False
           self.send_expression (item)

# --------------------------------------------------------------------------

   def send_inner_stat (self, param) :
       self.style_indent ()
       self.send_stat (param)
       self.style_unindent ()

   def send_stat_list (self, param) :
       if len (param.items) == 0 :
          self.send ("pass")
          self.style_new_line ()
       else :
          for item in param.items :
             self.send_stat (item)

   def send_empty_stat (self, param) :
       self.send ("pass")
       self.style_new_line ()

   def send_expression_stat (self, param) :
       self.send_expression (param.inner_expr)
       self.style_new_line ()

   def send_declaration_stat (self, param) :
       self.send_block_declaration (param.inner_decl)

   def send_compound_stat (self, param) :
       self.send_stat_list (param.body)

   def send_if_stat (self, param) :
       self.send ("if")
       self.send_expression (param.cond.cond_expr)
       self.send (":")
       self.style_new_line ()
       self.send_inner_stat (param.then_stat)
       if param.else_stat != None :
          self.send ("else")
          self.send (":")
          self.style_new_line ()
          self.send_inner_stat (param.else_stat)

   def send_while_stat (self, param) :
       self.send ("while")
       self.send_expression (param.cond.cond_expr)
       self.send (":")
       self.style_new_line ()
       self.send_inner_stat (param.body)

   def send_for_stat (self, param) :
       if param.from_expr != None :
          self.send_expression (param.from_expr.cond_expr)
       self.style_new_line ()
       self.send ("while")
       if param.to_expr != None :
          self.send_expression (param.to_expr)
       else :
          self.send ("True")
       self.send (":")
       self.style_new_line ()
       self.style_indent ()
       self.send_stat (param.body)
       if param.cond_expr != None :
          self.send_expression (param.cond_expr)
          self.style_new_line ()
       self.style_unindent ()

   def send_return_stat (self, param) :
       self.send ("return")
       if param.return_expr != None :
          self.send_expression (param.return_expr)
       self.style_new_line ()

# --------------------------------------------------------------------------

   def send_declaration_list (self, param) :
       for item in param.items :
          self.send_declaration (item)

   def send_type_specifiers (self, param) :
       if param.basic_name != None :
          self.send_qualified_name (param.basic_name)
       elif param.typename_name != None :
          self.send_qualified_name (param.typename_name)
       elif param.a_short or param.a_int or param.a_long :
          self.send ("int")
       elif param.a_bool :
          self.send ("bool")
       elif param.a_char == True :
          self.send ("char")
       elif param.a_wchar == True :
          self.send ("wchar_t")
       elif param.a_float == True :
          self.send ("float")
       elif param.a_double == True :
          self.send ("double")
       elif param.a_void == True :
          self.send ("void")

   def send_function_specifier (self, param) :
       self.send ("(")
       self.send_parameter_declaration_list (param.parameters)
       self.send (")")

   def send_parameter_declaration_list (self, param) :
       first = True
       for item in param.items :
           if not first :
              self.send (",")
           first = False
           self.send_parameter_declaration (item)

   def send_parameter_declaration (self, param) :
       if param.type != None :
          self.send_declarator (param.type.decl)
          if param.init != None :
             self.send ("=")
             self.send_expression (param.init)

   def send_simple_declaration (self, param) :
       inx = 0
       cnt = len (param.items)
       if param.func_body != None :
          self.openSection (param)
          self.send ("def")
          self.send_declarator (param.items [inx].decl)
          inx = inx + 1
          self.send (":")
          self.style_new_line ()
          self.style_indent ()
          self.send_stat_list (param.func_body.body)
          self.style_unindent ()
          self.style_empty_line ()
          self.closeSection ()
       else :
          while inx < cnt :
             self.send_simple_item (param.items [inx])
             inx = inx + 1

   def send_simple_item (self, param) :
       self.send_declarator (param.decl)
       if param.init != None :
          self.send_initializer (param.init)
       else :
          self.send ("=")
          self.send ("None")
       self.style_new_line ()

   def send_declarator (self, param) :
       if param.name != None :
          self.send_qualified_name (param.name)
       elif param.inner != None :
          self.send_declarator (param.inner)
       self.send_cont_specifier_list (param.cont)

   def send_cont_specifier_list (self, param) :
       for item in param.items :
          if isinstance (item, CmmFunctionSpecifier) :
             self.send_function_specifier (item)

   def send_type_id (self, param) :
       self.send_type_specifiers (param.type_spec)
       # self.send_abstract_declarator (param.decl)

   def send_initializer (self, param) :
       self.send ("=")
       self.send_initializer_item (param.value)

   def send_initializer_item (self, param) :
       if isinstance (param, CmmInitSimple) :
          self.send_expression (param.expr)

# --------------------------------------------------------------------------

   def send_class_declaration (self, param) :
       self.send ("class")
       self.send_simple_name (param.name)
       if param.members != None :
          if param.base_list != None :
             self.send ("(")
             self.send_base_specifier_list (param.base_list)
             self.send (")")
          self.send (":")
          self.style_new_line ()
          self.style_indent ()
          self.send_member_list (param.members)
          self.style_unindent ()
       else :
          self.send (":")
          self.style_new_line ()
          self.style_indent ()
          self.send ("pass")
          self.style_unindent ()

   def send_member_item (self, param) :
       if isinstance (param, CmmMemberVisibility) :
          pass
       elif isinstance (param, CmmSimpleDecl) :
          self.send_simple_declaration (param)

   def send_member_list (self, param) :
       if len (param.items) == 0 :
          self.send ("pass")
          self.style_new_line ()
       else :
          for item in param.items :
              self.send_member_item (item)

   def send_base_specifier_list (self, param) :
       first = False
       for item in param.items :
          if not first :
             self.send (",")
          first = False
          self.send_base_specifier (item)

   def send_base_specifier (self, param) :
       self.send_qualified_name (param.from_cls)

   def send_ctor_initializer (self, param) :
       for item in param.items :
          self.send_member_initializer (item)

   def send_member_initializer (self, param) :
       self.send_qualified_name (param.name)
       self.send ("=")
       self.send_expr_list (param.params)
       self.style_new_line ()

# --------------------------------------------------------------------------

   def send_empty_declaration (self, param) :
       pass

   def send_enum_declaration (self, param) :
       pass

   def send_namespace_declaration (self, param) :
       self.send_declaration_list (param.body)

# --------------------------------------------------------------------------

   def send_expression (self, param) :
       if param.kind == param.identExp:
          if param.name.id == "true" :
             self.send ("True")
          elif param.name.id == "false" :
             self.send ("False")
          else :
             self.send_qualified_name (param.name)
       if param.kind == param.numValueExp:
          self.send (param.value)
       if param.kind == param.charValueExp:
          self.send ("'" + param.value + "'")
       if param.kind == param.stringValueExp:
          inx = 0
          cnt = len (param.items)
          self.send ('"' + param.value + '"')
          while inx < cnt :
             self.send_string_literal_cont ('"' + param.items [inx].value + '"')
             inx = inx + 1
       if param.kind == param.thisExp:
          self.send ("this")
       if param.kind == param.subexprExp:
          self.send ("(")
          self.send_expression (param.param)
          self.send (")")
       if isinstance (param, CmmModernCastExpr) :
          if param.kind == param.DynamicCast :
             self.send ("dynamic_cast")
          elif param.kind == param.StaticCast :
             self.send ("static_cast")
          elif param.kind == param.ConstCast :
             self.send ("const_cast")
          elif param.kind == param.ReinterpreterCast :
             self.send ("reinterpret_cast")
          self.send ("<")
          self.send_type_id (param.type)
          self.send (">")
          self.send ("(")
          self.send_expression (param.param)
          self.send (")")
       if param.kind == param.typeidExp:
          self.send ("typeid")
          self.send ("(")
          if param.type != None :
             self.send_type_id (param.type)
          elif param.value != None :
             self.send_expression (param.value)
          self.send (")")
       if param.kind == param.typenameExp:
          self.send ("typename")
          self.send_qualified_name (param.name)
          self.send ("(")
          self.send_expr_list (param.list)
          self.send (")")
       if param.kind == param.typechangeExp:
          if param.a_signed == True :
             self.send ("signed")
          elif param.a_unsigned == True :
             self.send ("unsigned")
          elif param.a_short == True :
             self.send ("short")
          elif param.a_long == True :
             self.send ("long")
          elif param.a_bool == True :
             self.send ("bool")
          elif param.a_char == True :
             self.send ("char")
          elif param.a_wchar == True :
             self.send ("wchar_t")
          elif param.a_int == True :
             self.send ("int")
          elif param.a_float == True :
             self.send ("float")
          elif param.a_double == True :
             self.send ("double")
          elif param.a_void == True :
             self.send ("void")
          self.send ("(")
          self.send_expr_list (param.list)
          self.send (")")
       if param.kind == param.indexExp:
          self.send_expression (param.left)
          self.send ("[")
          self.send_expression (param.right)
          self.send ("]")
       if param.kind == param.callExp:
          self.send_expression (param.param)
          self.send ("(")
          self.send_expr_list (param.list)
          self.send (")")
       if param.kind == param.fieldExp:
          self.send_expression (param.param)
          self.style_no_space ()
          self.send (".")
          self.style_no_space ()
          self.send_qualified_name (param.name)
       if param.kind == param.ptrFieldExp:
          self.send_expression (param.param)
          self.style_no_space ()
          self.send (".")
          self.style_no_space ()
          self.send_qualified_name (param.name)
       if param.kind == param.postIncExp:
          self.send_expression (param.param)
          self.send ("++")
       if param.kind == param.postDecExp:
          self.send_expression (param.param)
          self.send ("--")
       if param.kind == param.incExp:
          self.send ("++")
          self.send_expression ()
       if param.kind == param.decExp:
          self.send ("--")
          self.send_expression (param.param)
       if param.kind == param.derefExp:
          self.send ("*")
          self.send_expression (param.param)
       if param.kind == param.addrExp:
          self.send ("&")
          self.send_expression (param.param)
       if param.kind == param.plusExp:
          self.send ("+")
          self.send_expression (param.param)
       if param.kind == param.minusExp:
          self.send ("-")
          self.send_expression (param.param)
       if param.kind == param.bitNotExp:
          self.send ("~")
          self.send_expression (param.param)
       if param.kind == param.logNotExp:
          self.send ("not")
          self.send_expression (param.param)
       if param.kind == param.scopeExp:
          self.send (".")
          self.send_expression (param.param)
       if param.kind == param.sizeofExp:
          self.send ("sizeof")
          if param.type != None :
             self.send ("(")
             self.send_type_id (param.type)
             self.send (")")
          elif param.value != None :
             self.send_expression (param.value)
       if param.kind == param.newExp:
          # send ("new")
          if param.type1 != None :
             self.send_type_id (param.type1)
          elif param.type2 != None :
             self.send ("(")
             self.send_type_id (param.type2)
             self.send (")")
          if param.init != None :
             self.send ("(")
             self.send_expr_list (param.init)
             self.send (")")
       if param.kind == param.deleteExp:
          self.send ("delete")
          if param.a_array == True :
             self.send ("[")
             self.send ("]")
          self.send_expression (param.param)
       if param.kind == param.dotMemberExp :
          self.send_expression (param.left)
          self.send (".*")
          self.send_expression (param.right)
       if param.kind == param.arrowMemberExp :
          self.send_expression (param.left)
          self.send ("->*")
          self.send_expression (param.right)
       if param.kind == param.mulExp :
          self.send_expression (param.left)
          self.send ("*")
          self.send_expression (param.right)
       if param.kind == param.divExp :
          self.send_expression (param.left)
          self.send ("/")
          self.send_expression (param.right)
       if param.kind == param.modExp :
          self.send_expression (param.left)
          self.send ("%")
          self.send_expression (param.right)
       if param.kind == param.addExp :
          self.send_expression (param.left)
          self.send ("+")
          self.send_expression (param.right)
       if param.kind == param.subExp :
          self.send_expression (param.left)
          self.send ("-")
          self.send_expression (param.right)
       if param.kind == param.shlExp :
          self.send_expression (param.left)
          self.send ("<<")
          self.send_expression (param.right)
       if param.kind == param.shrExp :
          self.send_expression (param.left)
          self.send (">>")
          self.send_expression (param.right)
       if param.kind == param.ltExp :
          self.send_expression (param.left)
          self.send ("<")
          self.send_expression (param.right)
       if param.kind == param.gtExp :
          self.send_expression (param.left)
          self.send (">")
          self.send_expression (param.right)
       if param.kind == param.leExp :
          self.send_expression (param.left)
          self.send ("<=")
          self.send_expression (param.right)
       if param.kind == param.geExp :
          self.send_expression (param.left)
          self.send (">=")
          self.send_expression (param.right)
       if param.kind == param.eqExp :
          self.send_expression (param.left)
          self.send ("==")
          self.send_expression (param.right)
       if param.kind == param.neExp :
          self.send_expression (param.left)
          self.send ("!=")
          self.send_expression (param.right)
       if param.kind == param.bitAndExp :
          self.send_expression (param.left)
          self.send ("&")
          self.send_expression (param.right)
       if param.kind == param.bitXorExp :
          self.send_expression (param.left)
          self.send ("^")
          self.send_expression (param.right)
       if param.kind == param.bitOrExp :
          self.send_expression (param.left)
          self.send ("|")
          self.send_expression (param.right)
       if param.kind == param.logAndExp :
          self.send_expression (param.left)
          self.send ("and")
          self.send_expression (param.right)
       if param.kind == param.logOrExp :
          self.send_expression (param.left)
          self.send ("or")
          self.send_expression (param.right)
       if param.kind == param.assignExp :
          self.send_expression (param.left)
          self.send ("=")
          self.send_expression (param.right)
       if param.kind == param.addAssignExp :
          self.send_expression (param.left)
          self.send ("+=")
          self.send_expression (param.right)
       if param.kind == param.subAssignExp :
          self.send_expression (param.left)
          self.send ("-=")
          self.send_expression (param.right)
       if param.kind == param.mulAssignExp :
          self.send_expression (param.left)
          self.send ("*=")
          self.send_expression (param.right)
       if param.kind == param.divAssignExp :
          self.send_expression (param.left)
          self.send ("/=")
          self.send_expression (param.right)
       if param.kind == param.modAssignExp :
          self.send_expression (param.left)
          self.send ("%=")
          self.send_expression (param.right)
       if param.kind == param.shlAssignExp :
          self.send_expression (param.left)
          self.send ("<<=")
          self.send_expression (param.right)
       if param.kind == param.shrAssignExp :
          self.send_expression (param.left)
          self.send (">>=")
          self.send_expression (param.right)
       if param.kind == param.andAssignExp :
          self.send_expression (param.left)
          self.send ("&=")
          self.send_expression (param.right)
       if param.kind == param.xorAssignExp :
          self.send_expression (param.left)
          self.send ("^=")
          self.send_expression (param.right)
       if param.kind == param.orAssignExp :
          self.send_expression (param.left)
          self.send ("|=")
          self.send_expression (param.right)
       if param.kind == param.condExp :
          self.send_expression (param.left)
          self.send ("?")
          self.send_expression (param.middle)
          self.send (":")
          self.send_expression (param.right)
       if param.kind == param.commaExp :
          self.send_expression (param.left)
          self.send (",")
          self.send_expression (param.right)
       if param.kind == param.throwExp:
          self.send ("throw")
          if param.expr != None :
             self.send_expression (param.expr)

# --------------------------------------------------------------------------

   def send_stat (self, param) :
       if param.stat == param.declarationStat :
          self.send_declaration_stat (param)
       elif param.stat == param.expressionStat :
          self.send_expression_stat (param)
       elif param.stat == param.emptyStat :
          self.send_empty_stat (param)
       elif param.stat == param.compoundStat :
          self.send_compound_stat (param)
       elif param.stat == param.caseStat :
          self.send_case_stat (param)
       elif param.stat == param.defaultStat :
          self.send_default_stat (param)
       elif param.stat == param.ifStat :
          self.send_if_stat (param)
       elif param.stat == param.switchStat :
          self.send_switch_stat (param)
       elif param.stat == param.whileStat :
          self.send_while_stat (param)
       elif param.stat == param.doStat :
          self.send_do_stat (param)
       elif param.stat == param.forStat :
          self.send_for_stat (param)
       elif param.stat == param.breakStat :
          self.send_break_stat (param)
       elif param.stat == param.continueStat :
          self.send_continue_stat (param)
       elif param.stat == param.returnStat :
          self.send_return_stat (param)
       elif param.stat == param.gotoStat :
          self.send_goto_stat (param)
       elif param.stat == param.tryStat :
          self.send_try_stat (param)

# --------------------------------------------------------------------------

   def send_declaration (self, param) :
       if isinstance (param, CmmSimpleDecl) :
          self.send_simple_declaration (param)
       elif isinstance (param, CmmEmptyDecl) :
          self.send_empty_declaration (param)
       elif isinstance (param, CmmClassDecl) :
          self.send_class_declaration (param)
       elif isinstance (param, CmmEnumDecl) :
          self.send_enum_declaration (param)
       elif isinstance (param, CmmTypedefDecl) :
          self.send_typedef_declaration (param)
       elif isinstance (param, CmmFriendDecl) :
          self.send_friend_declaration (param)
       elif isinstance (param, CmmNamespaceDecl) :
          self.send_namespace_declaration (param)
       elif isinstance (param, CmmExternDecl) :
          self.send_extern_declaration (param)
       elif isinstance (param, CmmUsingDecl) :
          self.send_using_declaration (param)
       # elif isinstance (param, CmmTemplateDecl) :
       #   send_template_declaration (param)

   def send_block_declaration (self, param) :
       if isinstance (param, CmmSimpleDecl) :
          self.send_simple_declaration (param)
       elif isinstance (param, CmmTypedefDecl) :
          self.send_typedef_declaration (param)
       elif isinstance (param, CmmUsingDecl) :
          self.send_using_declaration (param)

# --------------------------------------------------------------------------

# kate: indent-width 1; show-tabs true; replace-tabs true; remove-trailing-spaces all
